/*
--------------------------------------------------------------------------------

QuickGamma

Copyright (c) 2003 Randy Reddig
http://shaderlab.com/

Win32 tray hack to quickly set display gamma.

Portions of this code were based on the Tray42 example by Michael T. Smith
Copyright 1997 Michael T. Smith / R.A.M. Technology <aa529@chebucto.ns.ca>

Permission to use, copy, modify, and distribute this software and its
documentation for any purpose and without fee is hereby granted,
provided that the above copyright notice appear in all copies and that
both that copyright notice and this permission notice appear in
supporting documentation.

This file is provided AS IS with no warranties of any kind.	The author
shall have no liability with respect to the infringement of copyrights,
trade secrets or any patents by this file or any part thereof.	In no
event will the author be liable for any lost revenue or profits or
other special, indirect and consequential damages.

Changes:
Andreas "BlackHC" Kirsch [08.2006]
-commented out the idle priority setting to make it work perfectly when running games, etc, too
-reregister the notify window automatically after an explorer restart

--------------------------------------------------------------------------------
*/



/*
dependencies
*/

#include <windows.h>
#include <windowsx.h>
#include <shellapi.h>
#include <commctrl.h>
#include <afxres.h>
#include <math.h>
#include "resource.h"


/*
macros
*/

#define WINDOW_CLASSNAME	"QuickGamma"
#define UWM_SYSTRAY			(WM_USER + 1)
#define	ID_TRACKER			1313
#define ID_LABEL			1314
#define ID_READOUT			1315



/*
globals
*/

HINSTANCE					gInstance, gPrevInstance;
LPSTR						gArgs;
int							gShowWindow;

HFONT						gFont, gBoldFont;
WNDCLASSEX					gWindowClass;
HWND						gWindow, gTracker, gLabel, gReadout;
HMENU						gMenuBar, gMenu;
NOTIFYICONDATA				gNotifyIconData;

WORD						gGammaTable[ 768 ];

// global, registered event when a new taskbar is created
UINT						WM_TASKBAR_CREATED;

/*
code
*/

/*
CreateNotifyIcon()
creates the notify icon
*/

void CreateNotifyIcon(void)
{
	gNotifyIconData.cbSize = sizeof( NOTIFYICONDATA );
	gNotifyIconData.hWnd = gWindow;
	gNotifyIconData.uID = 1;
	gNotifyIconData.uFlags = NIF_MESSAGE | NIF_ICON | NIF_TIP;
	gNotifyIconData.uCallbackMessage = UWM_SYSTRAY;
	gNotifyIconData.hIcon = LoadImage( gInstance, MAKEINTRESOURCE( IDI_QUICKGAMMA ), IMAGE_ICON,
		GetSystemMetrics( SM_CXSMICON ), GetSystemMetrics( SM_CYSMICON ), 0 );
	strcpy( gNotifyIconData.szTip, "QuickGamma" );

	Shell_NotifyIcon( NIM_ADD, &gNotifyIconData );
}

/*
SetGammaTable()
sets the gamma table for a given gamma value, between 0.0 and 3.0+
*/

void SetGammaTable( double r, double g, double b )
{
	int		i;
	double	v;
	
	
	/* invert the values */
	if( r == 0.0 )
		r = 1.0;
	r = 1.0 / r;
	if( g == 0.0 )
		g = 1.0;
	g = 1.0 / g;
	if( b == 0.0 )
		b = 1.0;
	b = 1.0 / b;
	
	/* borrowed from opengl faq: http://talika.eii.us.es/~titan/oglfaq/ */
	for( i = 0; i < 256; i++ )
	{
		/* red */
		v = pow( (i + 1) / 256.0, r );
		if( v < 0 )
			v = 0;
		if( v > 1.0 )
			v = 1.0;
		gGammaTable[ i ] = v * 65535;
		
		/* green */
		v = pow( (i + 1) / 256.0, g );
		if( v < 0 )
			v = 0;
		if( v > 1.0 )
			v = 1.0;
		gGammaTable[ i + 256 ] = v * 65535;
		
		/* blue */
		v = pow( (i + 1) / 256.0, b );
		if( v < 0 )
			v = 0;
		if( v > 1.0 )
			v = 1.0;
		gGammaTable[ i + 512 ] = v * 65535;
	}
}



/*
WindowProc()
window procedure
*/

LRESULT CALLBACK WindowProc( HWND window, UINT message, WPARAM wParam, LPARAM lParam )
{
	int		x, y, w, h, sw, sh, value;
	double	gamma;
	char	temp[ 32 ];
	RECT	rect;
	POINT	pt;
	HDC		dc;

	/* handle custom id messages */
	/* handle creation of a new taskbar (e.g. after an explorer restart or crash) */
	if( message == WM_TASKBAR_CREATED ) {
		CreateNotifyIcon();
		return 0;
	}
	
	/* handle the message */
	switch( message )
	{
		/* handle exit */
		case WM_DESTROY:
			gNotifyIconData.cbSize = sizeof( NOTIFYICONDATA );
			gNotifyIconData.hWnd = window;
			gNotifyIconData.uID = 1;
			gNotifyIconData.uFlags = NIF_TIP;
			Shell_NotifyIcon( NIM_DELETE, &gNotifyIconData );
			PostQuitMessage( 0 );
			return 0;
		
		/* handle defocus */
		case WM_ACTIVATE:
			switch( wParam )
			{
				case WA_INACTIVE:
					ShowWindow( gWindow, FALSE );
					return 0;
			}
			break;
		
		/* handle tray icon activity */
		case UWM_SYSTRAY: // We are being notified of mouse activity over the icon
			switch( lParam )
			{
				/* left click */
				case WM_LBUTTONDOWN:
					/* position the window */
					GetWindowRect( gWindow, &rect );
					GetCursorPos( &pt );
					w = rect.right - rect.left;
					h = rect.bottom - rect.top;
					x = pt.x - w / 2;
					y = pt.y - h - 4;
					
					/* make sure it's onscreen */
					dc = GetDC( NULL );
					sw = GetDeviceCaps( dc, HORZRES );
					sh = GetDeviceCaps( dc, VERTRES );
					if( x < 8 )
						x = 8;
					if( y < 8 )
						y = 8;
					if( x > (sw - w - 8) )
						x = sw - w - 8;
					if( y > (sh - h - 8) )
						y = sh - h - 8;
					
					MoveWindow( gWindow, x, y, (rect.right - rect.left), (rect.bottom - rect.top), TRUE );
					
					/* show the window */
					ShowWindow( gWindow, TRUE );
					SetForegroundWindow( gWindow );
					break;
				
				/* left double-click */
				case WM_LBUTTONDBLCLK:
					/* set gamma to last known set value */
					//%	SetGammaTable( 1, 1, 1 );
					dc = GetDC( NULL );
					SetDeviceGammaRamp( dc, gGammaTable );
					break;
				
				/* right click */
				case WM_RBUTTONUP:
					/* win95 hack */
					SetForegroundWindow( window );
					
					/* track popup menu */
					GetCursorPos( &pt );
					switch( TrackPopupMenu( gMenu, TPM_RETURNCMD | TPM_RIGHTBUTTON, pt.x, pt.y, 0, window,	NULL ) )
					{
						case IDM_EXIT:
							DestroyWindow( window );
							break;
						case IDM_ABOUT:
							MessageBox( window, "QuickGamma 0.1.2\n\nCopyright (c) 2003 Randy Reddig - http://shaderlab.com/,\n\t (c) 2006 Andreas Kirsch - http://blackhc.wordpress.com/",
								"About", MB_TASKMODAL );
							break;
					}
					
					/* win95 hack pt. 2 */
					PostMessage( window, 0, 0, 0 );
					break;
			}
			return 0;
		
		/* handle tracker events */
		case WM_HSCROLL:
			/* get tracker value (gamma * 1000) */
			value = SendMessage( gTracker, TBM_GETPOS, 0, 0 );
			gamma = (double) value / 1000;
			
			/* set gamma readout */
			if( fabs( gamma - 1.0 ) < 0.01 )
			{
				gamma = 1.0;
				SetWindowFont( gReadout, gBoldFont, FALSE );
			}
			else
				SetWindowFont( gReadout, gFont, FALSE );
			sprintf( temp, "%1.2f", gamma );
			SetWindowText( gReadout, temp );
			
			/* set gamma */
			SetGammaTable( gamma, gamma, gamma );
			dc = GetDC( NULL );
			SetDeviceGammaRamp( dc, gGammaTable );
			break;
	}
	
	/* default handling */
	return DefWindowProc( window, message, wParam, lParam );
}



/*
WinMain()
main entry point
*/

int WINAPI WinMain( HINSTANCE instance, HINSTANCE prevInstance, LPSTR args, int showWindow )
{
	MSG			msg;
	RECT		rect;

	
	/* set globals */
	gInstance = instance;
	gPrevInstance = prevInstance;
	gArgs = args;
	gShowWindow = showWindow;
	
	/* set low priority */
	/*SetPriorityClass( GetCurrentProcess(), IDLE_PRIORITY_CLASS );*/
		
	/* get font */
	gFont = CreateFont( -11, 0, 0, 0, FW_NORMAL, FALSE, FALSE, FALSE, ANSI_CHARSET,
		OUT_DEFAULT_PRECIS, CLIP_DEFAULT_PRECIS, DEFAULT_QUALITY, (FF_DONTCARE | DEFAULT_PITCH), "Tahoma" );
	gBoldFont = CreateFont( -11, 0, 0, 0, FW_BOLD, FALSE, FALSE, FALSE, ANSI_CHARSET,
		OUT_DEFAULT_PRECIS, CLIP_DEFAULT_PRECIS, DEFAULT_QUALITY, (FF_DONTCARE | DEFAULT_PITCH), "Tahoma" );
	
	/* register window class */
	gWindowClass.cbSize = sizeof( WNDCLASSEX );
	gWindowClass.style = 0;
	gWindowClass.lpfnWndProc = WindowProc;
	gWindowClass.cbClsExtra = 0;
	gWindowClass.cbWndExtra = 0;
	gWindowClass.hInstance = gInstance;
	gWindowClass.hIcon = LoadIcon( gInstance, MAKEINTRESOURCE( IDI_QUICKGAMMA ) );
	gWindowClass.hCursor = LoadCursor( NULL, IDC_ARROW );
	gWindowClass.hbrBackground = (HBRUSH) (COLOR_WINDOW);
	gWindowClass.lpszMenuName = NULL;
	gWindowClass.lpszClassName = WINDOW_CLASSNAME;
	gWindowClass.hIconSm = LoadImage( gInstance, MAKEINTRESOURCE( IDI_QUICKGAMMA ), IMAGE_ICON,
		GetSystemMetrics( SM_CXSMICON ), GetSystemMetrics( SM_CYSMICON ), 0 );
	
	RegisterClassEx( &gWindowClass );

	/* get the shared "TaskbarCreated" window message id */
	WM_TASKBAR_CREATED = RegisterWindowMessage( "TaskbarCreated" );
	
	/* create window */
	SetRect( &rect, 0, 0, 160, 52 );
	AdjustWindowRectEx( &rect, (WS_POPUP | WS_DLGFRAME), FALSE, WS_EX_PALETTEWINDOW );
	
	gWindow = CreateWindowEx( WS_EX_PALETTEWINDOW, WINDOW_CLASSNAME, WINDOW_CLASSNAME, (WS_POPUP | WS_DLGFRAME),
		rect.left, rect.top, (rect.right - rect.left), (rect.bottom - rect.top),
		NULL, NULL, gInstance, NULL );
	
	/* create gamma tracker */
    gTracker = CreateWindowEx( 0, TRACKBAR_CLASS, "GammaTracker", (WS_CHILD | WS_VISIBLE), 
        2, 18, 156, 32, gWindow, ID_TRACKER, gInstance, NULL );
	
    SendMessage( gTracker, TBM_SETRANGE, (WPARAM) TRUE, (LPARAM) MAKELONG( 100, 3000 ) );
    SendMessage( gTracker, TBM_SETTIC, 0, (LPARAM) 1000 );
    SendMessage( gTracker, TBM_SETPOS, (WPARAM) TRUE, (LPARAM) 1000 );
    
    /* create labels */
	gLabel = CreateWindowEx( 0, WC_STATIC, "Gamma", (WS_CHILD | WS_VISIBLE | SS_LEFT), 
		2, 2, 78, 16, gWindow, ID_LABEL, gInstance, NULL );
	SetWindowFont( gLabel, gFont, TRUE );
    SetWindowText( gLabel, "Gamma" );
    gReadout = CreateWindowEx( 0, WC_STATIC, "Readout", (WS_CHILD | WS_VISIBLE | SS_RIGHT), 
		80, 2, 78, 16, gWindow, ID_READOUT, gInstance, NULL );
	SetWindowFont( gReadout, gBoldFont, FALSE );
    SetWindowText( gReadout, "1.00" );
	
	/* load menu */
	gMenuBar = LoadMenu( gInstance, MAKEINTRESOURCE( IDM_QUICKGAMMA ) );
	gMenu = GetSubMenu( gMenuBar, 0 );
	
	/* set up notifyicon */
	CreateNotifyIcon();
	
	/* get existing gamma table */
	GetDeviceGammaRamp( GetWindowDC( NULL ), gGammaTable );
	
	/* main loop */
	while( GetMessage( &msg, NULL, 0, 0 ) )
	{
		TranslateMessage( &msg );
		DispatchMessage( &msg );
	}
	
	/* exit */
	return msg.wParam;
}

